(function($, PHD, window, undefined) {

    var $form = $("#license-edit-form"),
        $btnAddLicense = $(".btn-add-license-info"),
        $btnUpgrade = $(".btn-upgrade-license");
    var LicenseForm,
        loadindicator,
        FQDN_REGEXP = /^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){4}$/,
        isrepeat = (typeof(isrepeat) != 'undefined') ? isrepeat : false;

    function initForm() {
        $btnAddLicense.on("click", handleSend);
        $btnUpgrade.on("click", handleUpgrade);

        $form.find(".definition").popover({
            appendTo: $form
        });

        LicenseForm = PHD.FormController($form, {})
            .validate({
                errorLabelContainer: ".license-dialog-error-list",
                errorContainer: ".license-dialog-errors",
                showErrors: function(errorMap, errorList) {
                    $(".license-dialog-errors").find(".summary")
                        .html(gettext("Correct the following errors:"));
                    this.defaultShowErrors();
                },
                rules: {
                    license_key_entered: {
                        required: true
                    },
                    feature_string_entered: {
                        required: true
                    }
                },
                messages: {
                    license_key_entered: {
                        required: gettext("License Key is required")
                    },
                    feature_string_entered:{
                        required: gettext("Feature String is required")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                PHD.currentDialog.wizard("close");
            });

        LicenseForm.submit = function() {
            if($(".btn-add-license-info").is(':visible')){
                return false;
            }
            var self = this;
            var url = "/api/license/";
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            url += sid;
            var persistData = {};
            persistData.key = $('#license_key_entered').val();
            persistData.feature_string = $('#feature_string_entered').val();
            persistData.expiration_date = $('#expiration_date').val();
            if($('#expiration_date').val() === "" ){
                persistData.expiration_date = "never";
            }
            persistData = JSON.stringify(persistData);
            var load = PHD.showLoadingIndicator($form);
            PHD.Ajax
                .put(url, persistData, load)
                .done(function(data) {
                   // self.$form.trigger("formsubmit", [data, self.$form, self]);
                    var result = data.result[0];
                    if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        console.log("SUCCESS");
                        PHD.Ajax.get(url)
                            .done(function(data){
                                if(data != undefined){
                                    $("#asset_tag").text(data.asset_tag);
                                    $("#license_name").text(data.name);
                                    $("#install_date").text(data.install_date);
                                    $("#expires").text(data.expiration_date);
                                    $("#feature_description").text(data.feature_string_description);
                                    $("#feature_string").text(data.feature_string);
                                    $("#license_key").text(data.key);
                                    $("#license_key_entered").val("");
                                    $('#feature_string_entered').val("");
                                    $('#expiration_date').val("");
                                }
                            });
                        handleSend();
                    }
                })
                .always(function() {
                    self.$buttons.enable();
                });
        }
        
        $('#edit-appliance').on("beforeactivationtab", function(event,ui) {
            if((ui.oldTab.text()) === " License "){
                var currentData = jQuery.extend(true,{}, ui.currentData);
                var editedDataObject =  jQuery.extend(true,{}, currentData);
                var iscompare = true;

                if(($('#license_key_entered').val() != "") ||
                   ($('#feature_string_entered').val() != "") ||
                   ($('#expiration_date').val() != "")){
                    iscompare = false;
                }

                if(iscompare == false){
                   event.preventDefault();
                   PHD.confirmDialog("<p>" + gettext("Do you want to save your changes?") + "</p>", 
                        {title: gettext("Saving Alert")},"Yes")
                    .done(function() {
                       isrepeat = false;
                       $('#edit-appliance').trigger('alertdialogyesclick',isrepeat);
                    })
                    .fail(function() {
                        isrepeat = true;
                        $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                    }) 
                }else{
                    isrepeat = true;
                    $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                }
            }
        });

    }

    function handleSend() {

        if($("#update_license").is(':visible')){
            $("#update_license").hide();
            $(".btn-add-license-info").show();
            $(".btn-upgrade-license").show();
        }
        else{
            $("#update_license").show();
            $(".btn-add-license-info").hide();
            $(".btn-upgrade-license").hide();
        }
    }

    $("#clear_date").on("click", function(){
        $("#expiration_date").datepicker('setDate', null);
    });

    function handleUpgrade() {
        var info_url = "/api/license/request/?sid=" + PHD.appliance_sid;
        var load = PHD.showLoadingIndicator('body', true, 'Gathering information for registration page......');
        var resp = PHD.Ajax.get(info_url, load);
        resp.done(function(data) {
            var upgrade_link;
            if (data.request.link !== undefined) {
                upgrade_link = data.request.link;
            }
            var licenseObj = data.request.registration;
            console.log("License object is " + JSON.stringify(licenseObj));
            upgrade_link.data = licenseObj;

            var upgradeForm = $('<form></form>');
            upgradeForm.attr('id', 'id_upgrade_form');
            upgradeForm.attr('action', upgrade_link);
            upgradeForm.attr('method', 'post');
            upgradeForm.attr('target', '_blank');
            for (name in licenseObj) {
                var licenseInput = $("<input>");
                licenseInput.attr('name', name);
                licenseInput.attr('value', licenseObj[name]);
                upgradeForm.append(licenseInput);
            }
            // append to body (required by some browsers.
            upgradeForm.appendTo('body').submit();
            setTimeout(function() {
                upgradeForm.remove();
            }, 2000);
        });

    }

    /*
     *   DOM Ready Function
     */
    $(function() {
            initForm();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);