//dialog.config-advancedBackupCopyEdit.js

(function($, PHD, window, undefined) {
    var $form = $("#advanced-backupcopy-edit-form"),
        BackupCopyEditForm,
        loadindicator,
        isMakeTargetSuccess = false,
        isBackupCopySource = false,
        portOption = { "min": 1, "max": 9999, "step": 1},
        maxBackupCopies = { "min": 1, "max": 50, "step": 1},
        NETWORK_IP_REGEXP = /^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){3}([0])$/,
        MASK_IP_REGEXP = /^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){4}$/,
        isrepeat = (typeof(isrepeat) != 'undefined') ? isrepeat : false;

    var show_replication_config = false;
    var show_openvpn_config = true;
    var initialMaxConcurrentValue = -1;

    function initForm() {
        $form.find(".definition").popover({
            appendTo: $form
        });

        console.log("Selected appliance is " + JSON.stringify(selectedAppliance));
        if (selectedAppliance.local == false) {
            $(".advanced-backupcopy-settings-openvpn").hide();
            show_openvpn_config = false;
            if (selectedAppliance.role == "Replication Source") {
                // shows in specialized dialog for non-managed.
                $(".advanced-backupcopy-settings-config").show();
                show_replication_config = true;
                $("#suspend-resume-copies").show();
            }
        } else {
            // local - show openvpn settings if not CE
            if (!isCE) {
                $(".advanced-backupcopy-settings-openvpn").show();
            } else {
                $(".advanced-backupcopy-settings-openvpn").hide();
            }
            // Do show settings and config for local if a replicating source.
            show_replication_config = true;
            $(".advanced-backupcopy-settings-config").show();
            $("#suspend-resume-copies").show();
        }

        if ((selectedAppliance.status != "suspended") && (selectedAppliance.status != "available")){
            $('#suspendBackupSelection').attr('disabled','disabled');
        }else{
            $('#suspendBackupSelection').removeAttr('disabled');
            $('#suspendBackupSelection').prop('checked', selectedAppliance.status == "suspended");
        }

        isMakeTargetSuccess = false;
        isBackupCopySource = false;
        $("#securePortSelection").spinner(portOption).val(1194);
        $("#spinMaxBackupCopies").spinner(maxBackupCopies);

        if (show_replication_config) {
            var configURL = "/api/replication/config/?sid=" + selectedAppliance.id;;
            var resp = PHD.Ajax.get(configURL, null);
            resp.done(function(data) {
                console.log("success loading conifg, data = " + JSON.stringify(data));
                initialMaxConcurrentValue = parseInt(data.config.max_concurrent);
                $("#spinMaxBackupCopies").val(initialMaxConcurrentValue);
                $("#dropdownQueueScheme").val(parseInt(data.config.queue_scheme));
                if ( initialMaxConcurrentValue == 0 && selectedAppliance.local) {
                    $('#suspendBackupSelection').prop('checked', true);
                }
            });
        }

        BackupCopyEditForm = PHD.FormController($form, {})
        .validate({
            errorLabelContainer: ".advanced-backupcopy-dialog-error-list",
            errorContainer: ".advanced-backupcopy-dialog-errors",
            showErrors: function(errorMap, errorList) {
                $(".advanced-backupcopy-dialog-errors").find(".summary")
                    .html(gettext("Correct the following errors:"));
                this.defaultShowErrors();
            },
            rules: {  
                secure_network: {
                    required: true,
                    pattern: NETWORK_IP_REGEXP
                },
                secure_netmask: {
                    pattern: MASK_IP_REGEXP
                }
            },
            messages: {
                secure_network: {
                    required: gettext("Secure network is required"),
                    pattern: gettext("Secure network is not a valid address")
                },
                secure_netmask:
                {
                    pattern: gettext("Secure netmask is not a valid address")
                }
            }
        })
        .on("formcancel", function(event) {
            PHD.hideLoadingIndicator(loadindicator);
            PHD.currentDialog.wizard("close");
        })
        .on("formsubmit", function(event, data) {
            PHD.hideLoadingIndicator(loadindicator);
            PHD.currentDialog.wizard("close");
            if(typeof data.result !== 'undefined' && data.result[0].code != AJAX_RESULT_SUCCESS) {
                    PHD.throwNotice(data);
            }
        });
        
       // EncryptionForm.submit = function(){
        $("#btnSave").click(function () {
            var self = this;
            var backupCopyData = {};
            var backupCopySourceData = {};
            var isMakeTargetSuccess = false;
            var isBackupCopySource = false;

            if ($form.valid()) {
                if(show_openvpn_config && $('#optimizedBackupcopySelection').prop("checked") == true){
                    backupCopyData.network = $('#txtSecureNetwork').val();

                    if(($('#txtSecureNetmask').val()) != ''){
                        backupCopyData.mask =  $('#txtSecureNetmask').val();
                    }

                    if(($('#securePortSelection').val()) != ''){
                        backupCopyData.port =  $('#securePortSelection').val();
                    }

                    // Make a target if not already one.
                    backupCopyData = JSON.stringify(backupCopyData);
                    var makeTargetUrl = "/api/systems/make_target";
                    
                    loadindicator = PHD.showLoadingIndicator("body", true, "Saving Backup Copy Configure Data...");

                    PHD.Ajax
                        .post(makeTargetUrl, backupCopyData, loadindicator)
                        .done(function(data) {
                            isMakeTargetSuccess = true;

                            if(isMakeTargetSuccess == true && isBackupCopySource == true){
                                PHD.hideLoadingIndicator(loadindicator);
                                PHD.currentDialog.wizard("close");
                            }

                            if(typeof data.result !== 'undefined' && data.result[0].code != AJAX_RESULT_SUCCESS) {
                                PHD.hideLoadingIndicator(loadindicator);
                                PHD.throwNotice(data);
                            }
                        });
                } else {
                    isMakeTargetSuccess = true;
                }

                // update max concurrent and queue scheme.
                if (show_replication_config) {

                    if((($('#spinMaxBackupCopies').val()) != '') && ($('#spinMaxBackupCopies').prop("disabled") != 'disabled')){
                        backupCopySourceData.max_concurrent = parseInt($('#spinMaxBackupCopies').val());
                    }
                    backupCopySourceData.queue_value = parseInt($("#dropdownQueueScheme").val());
                    backupCopySourceData = JSON.stringify(backupCopySourceData);
                    var sid = selectedAppliance.id;
                    var BackupCopySourceUrl = "/api/replication/config/?sid=" + sid;

                    PHD.Ajax
                        .put(BackupCopySourceUrl, backupCopySourceData, loadindicator)
                        .done(function(data) {
                            isBackupCopySource = true;

                            if(isMakeTargetSuccess == true && isBackupCopySource == true){
                                PHD.hideLoadingIndicator(loadindicator);
                                PHD.currentDialog.wizard("close");
                            }

                            if(typeof data.result !== 'undefined' && data.result[0].code != AJAX_RESULT_SUCCESS) {
                                PHD.hideLoadingIndicator(loadindicator);
                                PHD.throwNotice(data);
                            }
                        })
                } else {
                    isBackupCopySource = true;
                }
            }

            suspendOrResume(initialMaxConcurrentValue);
        });
        
        $('#edit-appliance').on("beforeactivationtab", function(event,ui) {
            if((ui.oldTab.text()) === " Backup Copy "){
                var currentData = jQuery.extend(true,{}, ui.currentData);
                var editedDataObject =  jQuery.extend(true,{}, currentData);

                if (editedDataObject.configuredata === undefined) {
                    editedDataObject.configuredata = {};
                }

                if($('#optimizedBackupcopySelection').prop("checked") == true){
                    editedDataObject.configuredata.is_configured = true;
                 }else{
                    editedDataObject.configuredata.is_configured = false;
                }
                
                if($('#suspendBackupSelection').prop("checked") == true){
                    editedDataObject.isSuspendBackupSelection = true;
                 }else{
                    editedDataObject.isSuspendBackupSelection = false;
                 }
                
                editedDataObject.configuredata.network = $('#txtSecureNetwork').val();
                editedDataObject.configuredata.mask =  $('#txtSecureNetmask').val();
                editedDataObject.configuredata.port =  parseInt($('#securePortSelection').val());
                editedDataObject.maxbackupcoies.max_concurrent = $('#spinMaxBackupCopies').val();
                editedDataObject.maxbackupcoies.queue_scheme = $('#dropdownQueueScheme').val();
                
                var iscompare = compareTwoObjects(currentData,editedDataObject);

                if(iscompare == false){
                   event.preventDefault();
                   PHD.confirmDialog("<p>" + gettext("Do you want to save your changes?") + "</p>", 
                        {title: gettext("Saving Alert")},"Yes")
                    .done(function() {
                       isrepeat = false;
                       $('#edit-appliance').trigger('alertdialogyesclick',isrepeat);
                    })
                    .fail(function() {
                        isrepeat = true;
                        $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                    }) 
                }else{
                    isrepeat = true;
                    $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                }
            }
        });
    }

    function suspendOrResume( initialValueForMaxConcurrent ) {

        var base_url = "/api/replication/";
        var suspendFromTargetURL = base_url +  "suspend/source/?sid=1";
        var resumeFromTargetURL = base_url + "resume/source/?sid=1";
        var suspendFromSourceURL = base_url + "suspend/target";
        var resumeFromSourceURL = base_url + "resume/target";
        var suspendURL, resumeURL;

        var wantToSuspend = $('#suspendBackupSelection').prop("checked") == true;
        var maxConcurrentWasInitiallyZero = initialValueForMaxConcurrent == 0;

        // Check state to see if want to change it.
        var currentlySuspended = selectedAppliance.status == "suspended";

        var arg={};
        if (selectedAppliance.local) {
            // Local Appliance and I am replicating.
            if (targetObject !== undefined) {
                arg.target_hostname = targetObject.name;
            } else {
                arg.target_hostname = "_all_";
            }
            suspendURL = suspendFromSourceURL;
            resumeURL = resumeFromSourceURL;
        } else {
            // Non-local, so either a manager or a target.
            if (localAppliance !== null && localAppliance.role != null && localAppliance.role.indexOf("Target") != -1) {
                // I am a target, suspend the source.
                arg.source_hostname = selectedAppliance.name;
                suspendURL = suspendFromTargetURL;
                resumeURL = resumeFromTargetURL;
            }
        }

        var changing = false;
        var url;
        if (wantToSuspend == true && currentlySuspended == false){
            url = suspendURL;
            changing = true;

        } else if (wantToSuspend == false && (currentlySuspended == true || (maxConcurrentWasInitiallyZero == true && selectedAppliance.local && $("#spinMaxBackupCopies").val() == 0) )) {
            url = resumeURL;
            changing = true;
        }

        if (changing) {
            //this.$buttons.disable();
            var data = JSON.stringify(arg);
            var resp = PHD.Ajax.put(url, data, null);

            resp.done(function(data) {
                var result = data.result[0];
                PHD.hideLoadingIndicator(load);
                if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                    $(document).trigger("editappliance");
                    PHD.currentDialog.wizard("close");
                } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                    console.log("FAILURE");
                }
            })
                .always(function() {
                    PHD.hideLoadingIndicator(loadindicator);
                });
        }

        return this;
    }
    
    $("input[name=optimizedbackupcopygroup]:checkbox").change(function () {
        if ($("#optimizedBackupcopySelection").prop("checked")) {
            $('#txtSecureNetwork').removeAttr('disabled');
            $('#txtSecureNetmask').removeAttr('disabled');
            $('#securePortSelection').removeAttr('disabled');
        }
        else {
            $('#txtSecureNetwork').attr('disabled','disabled');
            $('#txtSecureNetmask').attr('disabled','disabled');
            $('#securePortSelection').attr('disabled','disabled');
        }
    }); 
    
    /* DOM Ready Function */
    $(function() {
        initForm();
    });
    
})(jQuery, window.PHD || (window.PHD = {}), window);