angular.module('backupCopyMedia', ['ngDialog', 'angulartics', 'angulartics.google.analytics'])
.service('mediaService', ['$analytics', 'ngDialog', '$rootScope', function($analytics, ngDialog, $rootScope) {

    this.prepareTarget = function(target, sid, user_label, load) {
        $analytics && $analytics.eventTrack('Prepare Target', {  category: 'Configure', label: 'Target' });
        var url = '/api/archive/media/prepare/';
        var obj = {};
        var now = new Date();
        var label = (user_label == null || user_label == "") ? "tgt_" + now.getDate() + "_" + now.getHours() : user_label;
        obj.label = label;
        if (target) {
            url += target.name + '/?sid=' + sid;
            if (target.archive) {
                obj.mount = true;
            }
        }
        obj = JSON.stringify(obj);
        return PHD.Ajax.put(url, obj, load, this.handleMediaError);
    };

    this.getSets = function(target, sid, load) {
        $analytics && $analytics.eventTrack('Prepare Target', {  category: 'Configure', label: 'Target' });
        var url = '/api/archive/media/sets/';
        if (target) {
            url += target.name + '/?sid=' + sid;
        }
        return PHD.Ajax.get(url, null, this.handleMediaError);
    };

    this.handleMediaError = function(jqXHR, textStatus, errorThrown) {
        if(jqXHR.status === 500) {
            var data = jqXHR.responseJSON;
            var error = data;
            if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                error = data.result[0].message;
                console.log("The error is:" + error);
            }
            var scope = $rootScope.$new();
            ngDialog.open({
                dialogType:'Error',
                modelDialogId:'ErrorDialog',
                dialogMessage: error,
                scope:scope,
                onConfirmOkButtonClick:'onConfirmError()'
            });
            scope.onConfirmError = function () {
                ngDialog.close('ErrorDialog');
            };
        } else {
            PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
        }
    };

    this.validateForm = function(form) {
        var MAX_LABEL = 12;
        var NAME_REGEX = /^[a-zA-Z0-9\_]+$/;
        form.validate({
            errorLabelContainer: "#dialog-error-list",
            errorContainer: "#dialog-errors",
            showErrors: function(errorMap, errorList) {
                $("#dialog-errors").find(".summary")
                    .html(gettext("Correct the following errors:"));
                this.defaultShowErrors();
            },
            rules: {
                label: {
                    maxlength: MAX_LABEL,
                    pattern: NAME_REGEX
                }
            },
            messages: {
                label: {
                    pattern: "alphanumeric characters or underscores only"
                }
            }
        });
    };

    this.showHelp = function(form) {
        if (form !== null) {
            form.find(".definition").popover({
                appendTo: form
            });
        }
    };

}])
.controller('eraseMediaCtrl', ['$scope', '$rootScope', '$http','ngDialog', 'mediaService', function($scope, $rootScope, $http, ngDialog, mediaService) {

    $scope.target = $scope.ngDialogData.target;
    $scope.sid = $scope.ngDialogData.appliance_id;
    $scope.showSets = false;
    $scope.form = null;
    $scope.media_label = null;

    $scope.confirmErase = function(){
        if ($scope.form && $scope.form.valid()) {
            $scope.showSets = false;
            var load = PHD.showLoadingIndicator('body', true, "Erasing Target...");
            var resp = mediaService.prepareTarget($scope.target, $scope.sid, $scope.media_label, load);
            resp.done(function (data) {
                $scope.closeThisDialog();
            });
        }
    };

    $scope.getSets = function() {
        console.log($scope.target, $scope.sid);
        var resp = mediaService.getSets($scope.target, $scope.sid, null);
        resp.done(function (data) {
            var sets = data.sets;
            $scope.setCount = 0;
            if (sets !== undefined && sets.length > 0) {
                $scope.setCount = sets.length;
                $scope.firstSet = sets[0];
                $scope.lastSet = sets[sets.length - 1];
                $scope.dateRangeString = $scope.setCount > 0 ? ", ranging from " + $scope.firstSet.date + " to " + $scope.lastSet.date + "." : ".";
            }
            $scope.showSets = true;
            setTimeout(function () {
                $scope.$apply();
            }, 250);
        });
    };

    $scope.showLabelToolTip = function() {
        var form = $scope.form;
        mediaService.showHelp(form);
    };

    $scope.init = function(dlg) {
        if (dlg.name == 'erase-media-dialog') {
            $scope.form = angular.element("#confirm-erase-form");
            mediaService.validateForm($scope.form);
            $scope.showLabelToolTip();
        }
    };

    $scope.$on('ngDialog.opened', function(event, obj) { $scope.init(obj) });

}])
.controller('prepareMediaCtrl', ['$scope', '$rootScope', '$http','ngDialog', 'mediaService', function($scope, $rootScope, $http, ngDialog, mediaService) {

    $scope.target = $scope.ngDialogData.target;
    $scope.sid = $scope.ngDialogData.appliance_id;
    $scope.form = null;
    $scope.media_label = null;

    $scope.confirmPrepare = function(){
        if ($scope.form && $scope.form.valid()) {
            console.log($scope.target, $scope.sid);
            var load = PHD.showLoadingIndicator('body', true, "Preparing Target...");
            var resp = mediaService.prepareTarget($scope.target, $scope.sid, $scope.media_label, load);
            resp.done(function (data) {
                $scope.closeThisDialog();
            });
        }
    };

    $scope.showLabelToolTip = function() {
        var form = $scope.form;
        mediaService.showHelp(form);
    };

    $scope.init = function(dlg) {
        if (dlg.name == 'prepare-media-dialog') {
            $scope.form = angular.element("#confirm-prepare-form");
            mediaService.validateForm($scope.form);
            $scope.showLabelToolTip();
        }
    };

    $scope.$on('ngDialog.opened', function(event, obj) { $scope.init(obj) });

}]);