angular.module('credentials',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('credentialControl', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http, ngDialog) {

        var applianceSelect;
        var CredentialsTable = null;
        var allCredentials;
        var credentialRowData = null;
        var base_url = "/api/credentials/";
        var credentialEdit;
        $scope.selectedCredential = null;

        $scope.initCtrl = function(dlg){
            $scope.credential = {};
            switch (dlg.name){
                case 'Credential-Add':
                    credentialEdit = false;
                    break;
                case 'Credential-Edit':
                    credentialEdit = true;
                    $scope.editProcessing();
                    break;
                case 'Credential-Manager':
                    processDialog(PHD, window);
                    break;
                default:
                    // No action.
                    break;
            }

        };

        function handleError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.onConfirmWarning = function () {
            ngDialog.close('SelectCredential');
        };

        $scope.launchAddCredentials = function() {
            credentialEdit = false;
            ngDialog.open({
                template: 'app/configure/credentials/add-edit-credentials.html',
                scope: $scope,
                overlay:true,
                name: 'Credential-Add',
                ngDialogStyle:'width:500px; height:410px;',
                closeByDocument: false,
                closeByEscape: true,
                modelDialogId:'add-credentials-dialog'
            });
        };

        $scope.launchCredentialEditor = function() {
            if (typeof id != "undefined" && id != null){
                ngDialog.open({
                    template: 'app/configure/credentials/add-edit-credentials.html',
                    name: 'Credential-Edit',
                    scope: $scope,
                    overlay:true,
                    ngDialogStyle:'width:500px; height:410px;',
                    modelDialogId : 'edit-credentials-dialog',
                    closeByDocument: false,
                    closeByEscape: true,
                    showClose: false
                });
            } else {
                ngDialog.open({
                    dialogType:'Warning',
                    modelDialogId:'SelectCredential',
                    dialogMessage: 'Select a credential to edit.',
                    scope:$scope,
                    onConfirmOkButtonClick:'onConfirmWarning()'
                });
            }
        };

        $scope.editProcessing = function(){
            $("#add-edit-credentials_dialog_title").text("Edit Credential");
            $scope.credential.name = credentialname;
            $scope.credential.username = username;
            $scope.credential.password = password;
            $("#id_domain").val(domain);
            if(isDefault){
                $('#id_default').prop("checked", true);
            }
            else{
                $('#id_default').prop("checked", false);
            }
        };

        $scope.deleteCredential = function() {
            if ($scope.selectedCredential) {
                ngDialog.open({
                    dialogType: 'Confirmation',
                    dialogMessage: "Are you sure you want to delete the selected credential?",
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'DeleteCredentialConfirmation',
                    onConfirmOkButtonClick:'onConfirmDelete()'
                });
            } else {
                ngDialog.open({
                    modelDialogId:'NoCredentialSelected',
                    dialogType: 'Information',
                    dialogMessage: "No credential has been selected for deletion.",
                    scope: $scope,
                    overlay:true,
                    onConfirmOkButtonClick:'onConfirmNoneSelected()'
                });
            }
        };

        $scope.onConfirmNoneSelected = function() {
            ngDialog.close('NoCredentialSelected');
        };

        $scope.onConfirmDelete = function() {
            console.log("Delete Credential Confirmation") ;
            ngDialog.close('DeleteCredentialConfirmation');
            var base_url = "/api/credentials/";
            console.log("Credential ID being deleted : " + $scope.selectedCredential.id + " -- appliance : " + $scope.selectedCredential.system_id);
            base_url += $scope.selectedCredential.id + "/" ;
            resp = PHD.Ajax.delete(base_url + "?sid=" +  $scope.selectedCredential.system_id);
            resp.done(function(data){
                var result = data.result[0];
                if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                    $scope.selectedCredential = null;
                    refreshCredentialList();
                }else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                    ngDialog.open({
                        dialogType: 'ERROR',
                        dialogMessage: data,
                        overlay:true
                    });
                }
            });
        };

        $scope.addCredentials = function() {
            console.log("Adding credentials..");
            saveCredentials();
        };

        function refreshCredentialList() {
            var resp = PHD.Ajax.get(base_url+ "?sid=" + PHD.appliance_sid);
            resp.done(function(data){
                var credentialList = data.data;
                credentialList.forEach(function(item, index) {
                    item.id = item.credential_id;
                    item.system_id = item.sid;
                    if(item.is_default)
                        item.display_name = item.display_name+ " (default)";
                });
                allCredentials = credentialList;
                if (CredentialsTable) {
                    if (!$scope.credentialTableLoaded) {
                        CredentialsTable.load(allCredentials);
                        $scope.credentialTableLoaded = true;
                    } else {
                        CredentialsTable.update(allCredentials, true);
                    }
                }
            });
        }

        function processDialog(PHD, window) {

            $scope.credentialTableLoaded = false;

            var $el = $("#credentials-dialog");
            var $credentialsTable;

            var dtOptions = {
                multiselect: false,
                sortable: true,
                sortOnHeader: true,
                sortableCells: [true, true, true, true],
                sortOnLoad: true,
                sortingType : "string",
                cellClasses: ["long-text truncate", "long-text truncate", null, null, null, null],
                placeholder: gettext("There are no credentials")
            };

            $credentialsTable = $el.find(".credentials-table");
            CredentialsTable = PHD.DataTable($credentialsTable, dtOptions);

            refreshCredentialList();

            $credentialsTable.on("rowselect", "tr", function (event, dbid, $el, DataTable, DataRow) {
                credentialRowData = DataRow.data;
                id = credentialRowData.id;
                isDefault = credentialRowData.is_default;
                credentialname = credentialRowData.display_name;
                username = credentialRowData.username;
                password = credentialRowData.password;
                domain = credentialRowData.domain;
                $scope.selectedCredential = credentialRowData;
            });

            $credentialsTable.on("loadfinished updatefinished", function() {
                $credentialsTable.scrollTable({"resize": false});
		        $credentialsTable.sortTable();
            });

        }

        function saveCredentials() {
            var obj = {};
            var display_name = $("#id_display_name").val();
            if (display_name != "") {
                obj.display_name = display_name;
                if(display_name.indexOf("(default)") > -1){
                    var name = display_name;
                    name = name.replace('(default)','');
                    obj.display_name = name;
                }
                else{
                    obj.display_name = display_name;
                }
            }
            obj.username = $("#id_username").val();
            var password = $("#id_password").val();
            if (password != "") {
                obj.password = password;
            }
            var domain = $("#id_domain").val();
            obj.domain = domain;
            obj.is_default = $("#id_default").prop("checked");

            var resp;
            var url = "/api/credentials/";
            if (credentialEdit) {
                url += id + "/?sid=" + $scope.selectedCredential.system_id;
                obj = JSON.stringify(obj);
                resp = PHD.Ajax.put(url, obj, null, handleError);
            } else {
                var sid = PHD.appliance_sid !== undefined ? PHD.appliance_sid : 1;
                url += "?sid=" + sid;
                obj = JSON.stringify(obj);
                resp = PHD.Ajax.post(url, obj, null, handleError);
            }
            resp.done(function(data) {
                refreshCredentialList();
                if(credentialEdit){
                    ngDialog.close('edit-credentials-dialog');
                }
                else{
                    ngDialog.close('add-credentials-dialog');
                }
            });
        }

        $scope.$on('ngDialog.opened', function(event,obj) { $scope.initCtrl(obj) });

    }]);
