angular.module('edit_server',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('EditServerCtrl', ['$scope', '$rootScope', '$http', 'AssetService', 'ngDialog', function($scope, $rootScope, $http, AssetService, ngDialog) {

        var selectedCredential;
        var selectedCredentialID = -1;
        var associatedAppliance;
        var serverEdit = true;
        var selectedRetention,MinMaxRetentionName;
        var assetId;
        var isAssetEncrypted;
        var $formName;

        // Used to unbind the watched event on dialog close.
        var unbindHandler = null;

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                case 'Edit-Server':
                    $formName = "EditServerForm";
                    $scope.editProcessing($scope.ngDialogData);
                    processDialog(PHD, window);
                    break;
                case 'Edit-Assets':
                    $formName = "EditMultipleAssetsForm";
                    $scope.editAssetProcessing($scope.ngDialogData);
                    processDialog(PHD, window);
                    break;
            }
        };

        $scope.editProcessing = function(asset){
            $scope.assets = asset;
            assetId = asset.id;
            isAssetEncrypted = asset.is_encrypted;
            $("#retentionSection").show();
            $("#id_installAgentDiv").hide();
            $("#id_server").val(asset.name);
            $("#id_ip").val(asset.ip);
            $("#edit_object_id").text(asset.id);
            $("#id_enabled").prop('checked', asset.is_enabled);
            $("#id_encrypted").prop('checked', asset.is_encrypted);
            $('#id_encrypted').prop('disabled', true);
            selectedCredential = asset.credentials != undefined ? asset.credentials.credential_id : -1;
            selectedRetention = asset.retention != undefined ? asset.retention.id_retentionPolicy : 0;
            MinMaxRetentionName = asset.retention;
            associatedAppliance = asset.system_id;
            $scope.selectedSystemID = asset.system_id;
            /*
             * For iSeries disable edit of, ip, server, encryption as these are set by the profile (console menu), only allow retention and credentials.
             * If not iSeries, re-enable fields, tooltips, and determine encryption status.
             */
            if (asset.os_type == 'iSeries') {
                $scope.iSeriesAsset = true;
                $("#id_ip").prop('disabled', true);
                $("#id_server").prop('disabled', true);
                $("#credential_id").prop('disabled', true);
                var message = "iSeries asset name and address must be changed by editing the hosts file and using the menu-based console interface.";
                $("#host_name_tooltip").text(message);
                $("#ip_address_tooltip").text(message);
                $("#credentials_tooltip").text("iSeries asset credentials must be managed through the menu-based console interface.");
            } else {
                $scope.iSeriesAsset = false;
                $("#id_ip").prop('disabled', false);
                $("#id_server").prop('disabled', false);
                $("#credential_id").prop('disabled', false);
                $("#host_name_tooltip").text(asset.os_family == "Windows" ? HOSTNAME_ERROR_MESSAGE : HOSTNAME_NON_WINDOWS_ERROR_MESSAGE);
                $("#ip_address_tooltip").text("Enter either the IP address or a resolvable hostname of this system.");
                $("#credentials_tooltip").text("Enter administrative credentials.");
                AssetService.getApplianceEncryptionStatus(associatedAppliance, isAssetEncrypted, true);
            }
        };

        $scope.editAssetProcessing = function(assets){
            $scope.assets = assets;
            $("#retentionSection").show();
            $("#id_server").hide();
            if ($("#retention_information_text").find('p').length > 0){     
                $("p").remove();
            }
            $scope.checkFromSameSystem = $scope.isFromSameSystem(assets);
            //Check if selected asset/assets is/are associated with local system.
            if($scope.checkFromSameSystem){
                if(assets.length === 1) {           //If only one asset is selected and is a VM we have "Virtual host" e.g. esx server name, Xen server name, field shown in Dialog
                    if (assets[0].asset_type === "VMware:VM" || assets[0].asset_type === "Hyper-V:VM" || assets[0].asset_type === "Xen:VM") {
                        // TODO: SET VIRTUAL HOST NAME
                        // TODO: Don't we want to display the name of Hyper-V:VM ?
                        console.log("Single VM edit");
                        $("#id_description").text("Edit settings of " + assets[0].asset_type + " - " + assets[0].name);
                        $("#id_virtualHost").text(assets[0].parent_name);
                        associatedAppliance = assets[0].system_id;
                        AssetService.getApplianceEncryptionStatus(associatedAppliance, assets[0].is_encrypted, true);
                    }
                    else{
                        $("#id_virtualHost_info").hide();
                    }
                }
                else if(assets.length > 1)
                {
                    $("#id_virtualHost_info").hide();
                    $("#id_description").text("Edit settings of " + assets.length + " assets");
                }
                $("#details_box_multiple_assets").show();
                $scope.isEncryptionEnabled = $scope.checkEncryption(assets);
                // App-aware credentials are only supported for VMware VM's. So check if selected assets are VMware VM's. If yes, show app-aware checkbox.
                $scope.appAwareFlag = $scope.checkAppAware(assets);
                if($scope.appAwareFlag){
                    $("#credentials_is_allowed").prop("checked",true);
                    var isAppAware = $scope.isAppAware(assets);
                    if(isAppAware){
                        $("#id_appAware").prop("checked",true);
                    }
                    $("#App-Aware-Section").show();
                    $("#credentials_is_allowed").prop("checked",true);
                    $("#credential_id").enable();
                }
                else{
                    //If selected assets are not VMware VM's check if credentials are allowed to be shown for selected assets. Eg: Physical assets, Sharepoint, oracle instances.
                    $("#App-Aware-Section").hide();
                    $scope.physicalAssetsFlag = $scope.checkPhysical(assets);
                    $scope.applicationsForAuthentication = $scope.checkAppsAuthSupported(assets);
                    if($scope.physicalAssetsFlag || $scope.applicationsForAuthentication){
                        $("#credentials_is_allowed").prop("checked",true);
                        $("#credential_id").enable();
                    }
                    else{
                        $("#credentials_is_allowed").prop("checked",false);
                        $("#credential_id").disable();
                    }
                }
                selectedCredential = -1;
                // Now check if all selected assets have same credentials associated to them.
                var credential = $scope.checkCredentials(assets);
                if(credential){
                    var credentials = {};
                    credentials = assets[0].credentials;
                    selectedCredential = (credentials != null && credentials != undefined && credentials != "") ? credentials.credential_id : -1;
                }
                else{
                    selectedCredential = -1;
                }
                // Even if a single VMWare host is selected we do not show encryption checkbox in UI since we do not support encryption at VMware host(ESX,VCenter) level.
                $scope.checkVMwareHost =  $scope.isVMwareHost(assets);
                if($scope.checkVMwareHost){
                    $("#details_box_multiple_assets").hide();
                }
                else{
                    associatedAppliance = assets[0].system_id;
                    AssetService.getApplianceEncryptionStatus(associatedAppliance, $scope.isEncryptionEnabled, true);
                }
            }
            else{                                                       //If user selects asset from any other system do not allow any operation.
                $("#credentials_is_allowed").prop("checked",false);
                $("#credentials_is_allowed").disable();
                $("#credential_id").disable();
                $("#App-Aware-Section").hide();
            }
            // Now check retention
            selectedRetention =  0;
            $("#retention_is_allowed").prop("checked",true);
            var flag = $scope.checkRetention(assets);
            // TODO : Why are we setting null here? I am supposed to display the appliance when a single VMware:WM is being edited.
            //associatedAppliance = null;
        };

        $scope.isFromSameSystem = function(data) {
            var index,flag = true;
            var systemID = data[0].system_id;
            for (index = 0; index < data.length; ++index) {
                if(data[index].system_id !== systemID){
                    flag = false;
                    break;
                }
                else{
                    $scope.selectedSystemID = systemID;
                }
            }
            return flag;
        };

        $scope.checkRetention = function(data) {
            var index,flag = true, instances = {};
            if(typeof(data[0].type) !== "undefined"){
                if(data[0].type === "SQL Server" || data[0].type === "Exchange" || data[0].type === "Hyper-V" || data[0].type === "Oracle"
                    || data[0].type === "SharePoint" || data[0].type === "UCS Service Profile" || data[0].type === "NDMP Device"){
                    instances = data[0].children;
                    var commonRetention = instances[0].retention;
                    for (index = 0; index < instances.length; ++index){
                        if(instances[index].min_max_policy){
                            if(instances[index].retention !== commonRetention){
                                flag = false;
                                continue;
                            }
                        }
                        else{
                            flag = false;
                        }
                    }

                    if(data[0].type === "SQL Server" || data[0].type === "Oracle"){
                        console.log("SQL or ORACLE");
                        $(".retention_checkbox_text").text("DB Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings are applied to all databases associated with this application. Existing retention settings will be overwritten.</p>");
                    }
                    else if(data[0].type === "Hyper-V"){
                        console.log("HyperV");
                        $(".retention_checkbox_text").text("VM Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings are applied to all VMs associated with this virtual host. Existing retention settings will be overwritten.</p>");
                    }
                    else if(data[0].type === "Exchange"){
                        console.log("Exchange");
                        $(".retention_checkbox_text").text("Exchange Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings are applied to all databases associated with this Application. Existing retention settings will be overwritten.</p>");
                    }
                    else if(data[0].type === "SharePoint"){
                        console.log("SharePoint");
                        $(".retention_checkbox_text").text("SharePoint Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings will be applied to Sharepoint farm associated with this Application. Existing retention settings will be overwritten.</p>");
                    }
                    else if(data[0].type === "UCS Service Profile"){
                        console.log("UCS Service Profile");
                        $(".retention_checkbox_text").text("UCS Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings are applied to all profiles associated with this UCS. Existing retention settings will be overwritten.</p>");
                    }
                    else if(data[0].type === "NDMP Device"){
                        console.log("NDMP Device");
                        $(".retention_checkbox_text").text("NDMP Backup Retention");
                        $("#retention_information_text").append("<p>Retention settings are applied to all volumes associated with this NDMP device. Existing retention settings will be overwritten.</p>");
                    }

                    if(flag === true){
                        MinMaxRetentionName = commonRetention;
                        //Construct min-max policy array and add it to Application array so it can be used by min-max-config.js
                        var minmaxValues = {};
                        minmaxValues.retention_max = instances[0].min_max_policy.retention_max;
                        minmaxValues.retention_min = instances[0].min_max_policy.retention_min;
                        minmaxValues.legal_hold = instances[0].min_max_policy.legal_hold;
                        $scope.assets[0].min_max_policy = minmaxValues;
                    }
                    else{
                        MinMaxRetentionName = "Multiple";
                        //Construct min-max policy array with 0 values and add it to Application array so it can be used by min-max-config.js
                        var minmaxValues = {};
                        minmaxValues.retention_max = 0;
                        minmaxValues.retention_min = 0;
                        minmaxValues.legal_hold = 0;
                        $scope.assets[0].min_max_policy = minmaxValues;
                    }
                    $("#retention_is_allowed").prop("checked",true);
                    $("#id_retentionPolicy").val(MinMaxRetentionName);
                    /*
                    for min/max retention, the retention policy name will be a disabled text box.
                    var retentionPolicySelect = $("#id_retentionPolicy");
                    retentionPolicySelect = retentionPolicySelect[0];
                    retentionPolicySelect.options[0] = new Option(MinMaxRetentionName, 0);
                    */
                }
                else{
                    for (index = 0; index < data.length; ++index) {
                        if(data[index].min_max_policy){
                            var min = data[0].retention;
                            if(data[index].retention !== min){
                                flag = false;
                                continue;
                            }
                        }
                        else{
                            flag = false;
                        }
                    }
                    if(flag === true){
                        MinMaxRetentionName = data[0].retention;
                    }
                    else{
                        MinMaxRetentionName = "Multiple";
                    }
                }
            }
            return flag;
        };

        $scope.isVMwareHost = function(data) {
            var index,flag = true;
            for (index = 0; index < data.length; ++index) {
                if(data[index].asset_type !== "VMware Host"){
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        $scope.checkAppAware = function(data) {
            var index,flag = true;
            for (index = 0; index < data.length; ++index) {
                if(data[index].asset_type !== "VMware:VM"){
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        $scope.checkPhysical = function(data) {
            var index,flag = true;
            for (index = 0; index < data.length; ++index) {
                if(data[index].asset_type !== "physical"){
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        $scope.isAppAware = function(data) {
            var index,flag = true;
            for (index = 0; index < data.length; ++index) {
                if(!data[index].app_aware){
                    flag = false;
                    continue;
                }
            }
            return flag;
        };
        $scope.checkCredentials = function(data) {
            var index, flag = true;
            for (index = 0; index < data.length; ++index) {
                var cred = data[0].credential_display;
                if(data[index].credential_display !== cred){
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        $scope.checkAppsAuthSupported = function(data) {
            var index,flag;
            for (index = 0; index < data.length; ++index) {
                if(data[index].asset_type === "Oracle" || data[index].asset_type === "SharePoint" || data[index].asset_type === "VMware:VM") {
                    flag = true;
                }
                else{
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        $scope.checkEncryption = function(data) {
            var index,flag;
            for (index = 0; index < data.length; ++index) {
                if(data[index].is_encrypted) {
                    flag = true;
                }
                else{
                    flag = false;
                    continue;
                }
            }
            return flag;
        };

        function processDialog(PHD, window) {

            var $form = $("form[name=" + $formName + "]");
            $scope.$form = $form;

            var credentials = [];
            var credentialsSelect;
            var $credentialsSelect = $("#credential_id");
            var applianceSelect;
            var $applianceSelect = $("#id_system");
            applianceSelect = $applianceSelect[0];
            applianceSelect.options.length = 0;
            var retentionPolicySelect;

            $scope.ipPattern = /^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$/;
            $scope.ipErrorString = "Enter valid IP address";

            $scope.credentialNamePattern = /^[ A-Za-z0-9_@./#&+-/!$%^*()=\\|:;'"<>,?`~{}\[\]]{0,64}$/;
            $scope.credentialNameErrorString = "Credential Name: Max 64 characters";

            var editServerForm = PHD.FormController($form, {
                serialize: function ($form) {
                    return $form.find("input").serialize();
                }
            });

            if ($formName == "EditServerForm") {
                editServerForm
                    .validate({
                        rules: {
                            ip: {
                                required: true,
                                pattern: !$scope.grandClient ? $scope.ipPattern : "N/A"
                            },
                            display_name: {
                                required: true,
                                pattern: $scope.credentialNamePattern
                            },
                            user_name: {
                                required: true
                            }
                        },
                        messages: {
                            ip: {
                                required: gettext("IP Address is required"),
                                pattern: gettext($scope.ipErrorString)
                            },
                            display_name: {
                                required: gettext("Credential name is required"),
                                pattern: gettext($scope.credentialNameErrorString)
                            },
                            user_name: {
                                required: gettext("Username is required")
                            }
                        }
                    });
            }

            editServerForm.submit = function() {
                if($formName == "EditMultipleAssetsForm")
                    $scope.editMultipleAssets();
                else
                    $scope.editAsset();
            };

            function refreshSystemsList() {
                AssetService.getSystemsList().done(function (data) {
                    var appliances = data.appliance.sort(AssetService.sortIDs);
                    appliances.forEach(function (system, index) {
                        applianceSelect.options[index] = new Option(system.name, system.id);
                    });
                });
            }

            function populateCredentials() {
                credentialsSelect = $credentialsSelect[0];
                credentialsSelect.options.length = 0;
                var i = 0;
                credentialsSelect.options[i] = new Option('None', -1);
                if(!$scope.appAwareFlag){
                    i++;
                    credentialsSelect.options[i] = new Option('New', 0);
                }
                if (credentials.length > 0) {
                    credentials.forEach(function (credential, index) {
                        if (credential.display !== null && credential.display !== undefined && credential.display.length > 0) {
                            credentialsSelect.options[index + (i+1)] = new Option(credential.display, credential.credential_id);
                        }
                    })
                }
            };

            function refreshRetentionList () {
                return PHD.Ajax.get('/api/retention/policy/', null, this.handlePolicyError);
            };

            $credentialsSelect.on("change", function (event) {
                selectedCredentialID = $credentialsSelect.val();
                $scope.selectedCredentialName =  $("#credential_id option:selected").text();
                if (selectedCredentialID == 0) {
                    $(".modal-form-credential" ).show();
                    $("#id_credentials_displayName").val("");
                    $("#id_credentials_username").val("");
                    $("#id_credentials_password").val("");
                    $("#id_domain").val("");
                } else {
                    $( ".modal-form-credential" ).hide();
                }
            });

            function initForm() {
                $form.find(".definition").popover({
                    appendTo: $form
                });

                refreshSystemsList();

                AssetService.getCredentials($scope.selectedSystemID).done(function (data) {
                    console.log("GET credentials");
                    credentials = data.data;
                    credentials.forEach(function (credential, index) {
                        if(credential.display_name != undefined){
                            var display = credential.display_name;
                            if (credential.is_default) {
                                display += " (Default)";
                            }
                            credential.display = display;
                        }
                        else{
                            credentials.splice(credentials.indexOf(credential),1);
                        }
                    });

                    populateCredentials();
                    $(".modal-form-credential").hide();
                    $("#credential_id").val(parseInt(selectedCredential));
                });

                var strategy = PHD.Ajax.get('/api/retention/strategy/', null, handlePolicyError);
                strategy.done(function(data){
                    retentionPolicySelect = $("#id_retentionPolicy");
                    retentionPolicySelect = retentionPolicySelect[0];
                    var i = 0;
                    if(data.data.strategy === "MinMax"){
                        //retentionPolicySelect.options[i] = new Option(MinMaxRetentionName, 0);
                        $("#id_retentionPolicy").val(MinMaxRetentionName);
                        $rootScope.retentionStrategy = "MinMax";
                    }
                    else{
                        $rootScope.retentionStrategy = "GFS";
                        retentionPolicySelect.options[i] = new Option('None', -1);
                        i++;
                        refreshRetentionList().done(function(data){
                            $scope.isPolicyGFS = true;
                            if(data != null && data != undefined)
                                data.data.forEach(function (system, index) {
                                    retentionPolicySelect.options[index+i] = new Option(system.name, system.id);
                                });
                        });
                        $("#id_system").val(parseInt(associatedAppliance));
                        $("#id_retentionPolicy").val(parseInt(selectedRetention));
                        $("#id_server").prop("disabled", true);
                    }
                });
                //AssetService.getApplianceEncryptionStatus(associatedAppliance, isAssetEncrypted, true);
            }
            initForm();

            $scope.launchRetentionManager = function () {
                if($rootScope.retentionStrategy === "MinMax"){
                    ngDialog.open({
                        template: 'app/configure/retention/min-max.html',
                        overlay: true,
                        scope: $scope,
                        data: $scope.assets,
                        name: 'MinMaxRetention',
                        ngDialogStyle:'width:600px;',
                        closeByDocument: false,
                        closeByEscape: false
                    });
                }
                else{
                    ngDialog.open({     //Gives error currently
                        template: 'app/configure/retention/retentionpolicy.html',
                        scope: $scope,
                        overlay:true,
                        name: 'Retention-Policy',
                        ngDialogStyle:'width:300px;',
                        // GFS TODO - enlarge dialog once we have weekly/monthly/yearly settings
                        //ngDialogStyle:'width:700px; height:400px;',
                        closeByDocument: false,
                        closeByEscape: false
                    });
                }

            };

            $scope.editMultipleAssets = function () {
                selectedCredentialID = $credentialsSelect.val();
                console.log("editMultipleAssets - serverEdit : "+ serverEdit);

                var obj = {};
                var credentials = {};

                console.log("selected = " + selectedCredentialID);
                if(selectedCredentialID == 0){
                    credentials.display_name = $("#id_credentials_displayName").val();
                    credentials.username = $("#id_credentials_username").val();
                    credentials.password = $("#id_credentials_password").val();
                    credentials.domain = $("#id_domain").val();
                    credentials.is_default = $("#id_default").prop("checked");
                    obj.credentials = credentials;
                }
                else if(selectedCredentialID == -1){
                    obj.credential_id = -1;
                }
                else{
                    obj.credential_id = parseInt(selectedCredentialID);
                }

                var client_url = "/api/credentials/bind/client/";
                var instance_url = "/api/credentials/bind/instance/";
                var encrypt_enable_url = "/api/encryption/enable-instance/";
                var encrypt_disable_url = "/api/encryption/disable-instance/";
                var putObj = JSON.stringify(obj);
                var hasErrors = false;

                if($("#id_encrypted").is(':visible') && $("#id_encrypted").is(':enabled')){
                    $scope.LoadIndicator = PHD.showLoadingIndicator("body", true, "Saving...");

                    obj = {};
                    var instance_ids = "";
                    $scope.assets.forEach(function(asset){
                        if($scope.physicalAssetsFlag && asset.file_level_instance_id){
                            instance_ids += asset.file_level_instance_id + ",";
                        }
                        else{
                            instance_ids += asset.id + ",";
                        }
                    });
                    var str = instance_ids.substring(0, instance_ids.length - 1);
                    obj.instance_id = str;
                    putObj = JSON.stringify(obj);
                    var encrypt_url = $("#id_encrypted").prop("checked") ? encrypt_enable_url : encrypt_disable_url;

                    var resp = PHD.Ajax.put(encrypt_url + "?sid=" + $scope.selectedSystemID, putObj, $scope.LoadIndicator, null);
                    resp.done(function(data) {
                        PHD.hideLoadingIndicator($scope.LoadIndicator);
                        if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                            var error = data.result[0].message;
                            console.log("The error " +
                            "is:" + error);
                            if (error != undefined) {
                                hasErrors = true;
                            }
                        }
                        if (!hasErrors && !$("#credentials_is_allowed").is(':checked')) {
                            $scope.closeAssetDialog(true);
                        }
                    });
                }

                if($("#credentials_is_allowed").is(':visible') && $("#credentials_is_allowed").is(':checked')){
                    console.log("selectedCredentialID "+selectedCredentialID);
                    if ($("#id_appAware").prop("checked") && (parseInt(selectedCredentialID) == -1 || selectedCredentialID == NaN || selectedCredentialID == null)) {
                        ngDialog.open({
                            dialogType:'WARNING',
                            modelDialogId:'EditAssetErrorDialog',
                            scope:$scope,
                            dialogMessage: 'To enable Application Aware VM Processing, you must add a new credential or select an existing one.',
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                        return false;
                    }
                    $scope.LoadIndicator2 = PHD.showLoadingIndicator("body", true, "Saving...");
                    if( $scope.appAwareFlag || $scope.applicationsForAuthentication){
                        obj = {};
                        var instance_ids="";
                        $scope.assets.forEach(function(asset){
                            instance_ids += asset.id + ",";
                        });
                        var str = instance_ids.substring(0, instance_ids.length - 1);
                        obj.instance_ids = str;
                        obj.app_aware = $("#id_appAware").prop("checked");

                        putObj = JSON.stringify(obj);
                        var resp = PHD.Ajax.put(instance_url + parseInt(selectedCredentialID) + "/?sid=" + $scope.selectedSystemID, putObj, $scope.LoadIndicator2, null);

                        resp.done(function(data) {
                            PHD.hideLoadingIndicator($scope.LoadIndicator2);
                            if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                                var error = data.result[0].message;
                                console.log("The error is:" + error);
                                if (error != undefined) {
                                    hasErrors = true;
                                }
                            }
                            if (!hasErrors) {
                                $scope.closeAssetDialog(true);
                            }
                        });
                    }
                    else if($scope.physicalAssetsFlag){
                        obj = {};
                        var clientInfo = {};
                        var i=0;
                        $scope.assets.forEach(function(asset){
                            var client = {};
                            client['clientID'] = asset.id;
                            client['name'] = asset.name;
                            client['credential_display'] = $scope.selectedCredentialName;
                            clientInfo[i] = client;
                            i++;
                        });
                        obj.clients = clientInfo;
                        parseInt(selectedCredentialID);
                        putObj = JSON.stringify(obj);
                        var resp = PHD.Ajax.put(client_url + parseInt(selectedCredentialID) + "/?sid=" + $scope.selectedSystemID, putObj, $scope.LoadIndicator2, handlePhysicalAssetsError);

                        resp.done(function(data) {
                            PHD.hideLoadingIndicator($scope.LoadIndicator2);
                            if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                                var error = data.result[0].message;
                                console.log("The error is:" + error);
                                if (error != undefined) {
                                    hasErrors = true;
                                }
                            }
                            if (!hasErrors) {
                                $scope.closeAssetDialog(true);
                            }
                        });
                    }

                }

               /* $scope.assets.forEach(function(asset) {
                    var resp;
                    if (asset.asset_type === ASSET_TYPE_PHYSICAL) {
                        obj.clientID = asset.id;
                        putObj = JSON.stringify(obj);
                        resp = PHD.Ajax.put(client_url + obj.credential_id + "/?sid=" + asset.system_id, putObj, null, null);
                    } else {
                        obj = {};
                        obj.credential_id = parseInt(selectedCredentialID);
                        obj.app_aware = true; // TODO, add to dialog for a VM.
                        putObj = JSON.stringify(obj);
                        resp = PHD.Ajax.put(instance_url + asset.id + "/?sid=" + asset.system_id, putObj, null, null);
                    }

                    resp.done(function(data) {
                        if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                            var error = data.result[0].message;
                            console.log("The error is:" + error);
                            if (error != undefined) {
                                hasErrors = true;
                            }
                        }
                    });
                });*/


            };

            $scope.onConfirmError = function () {
                ngDialog.close('EditAssetErrorDialog');
            };

            $scope.editAsset = function () {
                if ((!$scope.$form.valid())) {
                    return false;
                }
                console.log("Save asset clicked - serverEdit : "+ serverEdit);
                // Do not call PUT /api/clients for iSeries assets, just close the window.
                if ($scope.iSeriesAsset) {
                    $scope.closeAssetDialog();
                    return true;
                }
                var obj={};
                // associatedAppliance is being passed as query param. Form element(id_system) is not used by the API.
                //var associatedAppliance = $("#id_system").val();
                var selectedCredentialNow = $credentialsSelect.val();
                obj.name = $("#id_server").val();
                obj.os_type = ($scope.assets !== undefined && $scope.assets.os_family !== undefined) ? $scope.assets.os_family : "";
                obj.priority = 300;
                obj.is_enabled = true;
                obj.is_synchable = false;
                obj.use_ssl = false;
                obj.is_auth_enabled = (selectedCredentialNow == -1) ? false : true;
                obj.is_encrypted = $("#id_encrypted").prop("checked") ? true : false;

                var hostinfo = {};
                if (!$scope.grandClient) {
                    hostinfo.ip = $("#id_ip").val();
                }

                console.log("selected = " + selectedCredentialNow);
                if(selectedCredentialNow == 0){
                    var credentials = {};
                    credentials.display_name = $("#id_credentials_displayName").val();
                    credentials.username = $("#id_credentials_username").val();
                    credentials.password = $("#id_credentials_password").val();
                    credentials.domain = $("#id_domain").val();
                    credentials.is_default = $("#id_default").prop("checked");
                    obj.credentials = credentials;
                }
                else if(selectedCredentialNow == -1){
                    obj.credential_id = -1;
                }
                else{
                    obj.credential_id = parseInt(selectedCredentialNow);
                }
                obj.host_info = hostinfo;

                var base_url = "/api/clients/";

                obj = JSON.stringify(obj);
                console.log("selected retentionPolicy = " + $("#id_retentionPolicy").val());
                //TODO: VALIDATE the retentionID usage once the API is ready to attach the retention policy with the Asset.
                obj.retention_id = $("#id_retentionPolicy").val();
                console.log("PUT called to edit Asset with ID : " + assetId + " -- associatedAppliance : " + associatedAppliance);
                base_url +=assetId + "/";
                $scope.LoadIndicator = PHD.showLoadingIndicator("body", true, "Editing ...");
                resp = PHD.Ajax.put(base_url + "?sid=" +  associatedAppliance, obj, null, handleAssetError);

                resp.done(function(data) {
                    PHD.hideLoadingIndicator($scope.LoadIndicator);
                    $scope.closeAssetDialog();
                    var result = data.result;
                    if (Array.isArray(result)) {
                        result = result[0];
                    }
                    if (result !== undefined) {
                        if (parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                            ngDialog.close("Info-Error-Dialog");
                            console.log("SUCCESS");
                            $(document).trigger("assetChange");
                        } else if (parseInt(result.code) === AJAX_RESULT_ERROR) {
                            console.log("FAILURE");
                            ngDialog.open({
                                dialogType: 'ERROR',
                                modelDialogId:'Info-Error-Dialog',
                                dialogMessage: data,
                                overlay: true
                            });
                        } else {
                            console.log("SUCCESS");
                            $(document).trigger("assetChange");
                        }
                    }
                });
            };
        }

        function handlePhysicalAssetsError(jqXHR, textStatus, errorThrown){
            PHD.hideLoadingIndicator($scope.LoadIndicator);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.messages !== undefined) {
                    var errors = data.messages+"";
                    console.log(errors);
                    var errorArray = {};
                    errorArray = errors.split('|');
                    var index = errorArray.indexOf("");
                    errorArray.splice(index,1);
                    console.log(errorArray);
                    ngDialog.open({
                        template: 'app/configure/environments/MultipleErrorDialog.html',
                        overlay:true,
                        ngDialogStyle:'width:600px;',
                        closeByDocument: false,
                        closeByEscape: false,
                        data: errorArray,
                        modelDialogId:'ErrorDialog',
                        scope:$scope
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        };

        function handleAssetError (jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    PHD.hideLoadingIndicator($scope.LoadIndicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        };

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        function handlePolicyError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined &&
                    data.result[0].message.indexOf("strategy of the system is not GFS") != -1) {
                    console.log("Not GFS, don't allow assignment of policies");
                    $scope.isPolicyGFS = false;
                    $('#retentionSection').prop('disabled', true);
                    $('.retentionPolicyList').prop('disabled', true);
                    $('.retentionPolicyList').attr('title', data.result[0].message);
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        // When closing the asset dialog, unbind the ngDialog listener and if the asset was changed, trigger a reload.
        $scope.closeAssetDialog = function(triggerChange) {
            var changed = triggerChange || false;
            $scope.closeThisDialog();
            if ( _.isFunction(unbindHandler)) {
                unbindHandler();
            }
            if (changed) {
                $(document).trigger("assetChange");
            }
        };

        // Initialize the controller, returning the function to turn off the listener.
        unbindHandler = $scope.$on('ngDialog.opened', function(event,obj) {
            $scope.initCtrl(obj);
        });

        $(document).one('retentionChange', function(event, obj) {
            $scope.closeAssetDialog(true);
        });
    }]);
