(function($, PHD, window, undefined) {

    var $form = $("#form-server"),
        ServerForm,
        FQDN_REGEXP = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])(\:[0-9]{1,5})?$/i,
        NAME_REGEXP = /^[a-zA-Z0-9_-]*$/,
        $port = $("#id_port"),
        portSpinnerOptions = {"min": 1, "max": 65536, "step": 1},
        $protocol = $("#id_protocol");
    var applianceSelect = $("#id_system")[0];

    function initForm() {

        $form.find(".definition").popover({
            appendTo: $form
        });

        if(typeof EditDisabled !== 'undefined'){
            if(EditDisabled === true){
                $form.find("#id_name").disable();
                $form.find("#id_ip").disable();
                $form.find("#id_share").disable();
                $form.find("#id_system").disable();
                $("#host_name_tooltip").text("The name of the NAS cannot be changed.");
                $("#share_name_tooltip").text("The share name of the NAS cannot be changed.");
                $("#ip_tooltip").text("The IP address or hostname of the NAS cannot be changed.");
                EditDisabled = false;
            }
        }

        $port.spinner(portSpinnerOptions);

        ServerForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true,
                        pattern: NAME_REGEXP
                    },
                    "properties.hostname": {
                        required: true,
                        pattern: FQDN_REGEXP
                    },
                    "properties.share_name": {
                        required: true
                    }
                },
                messages: {
                    name: {
                        required: gettext("NAS name is required"),
                        pattern: gettext("NAS name cannot have space.")
                    },
                    "properties.hostname": {
                        required: gettext("Hostname or IP address is required"),
                        pattern: gettext("Hostname or IP address is not valid")
                    },
                    "properties.share_name": {
                        required: gettext("Share name is required")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
				if(resetTour){
					refreshPage();
					setTimeout(function(){
					var $scope = $("[ng-controller=GlobalCtrl]").scope();
					$scope.user.showTour = true;
					window.location.href = "#/dashboard/"; 
					resetTour = false;
					$(".btn-pref-cancel").trigger('click');
					}, 1000);
				}
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("assetChange");
                PHD.currentDialog.wizard("close");

                var scope = angular.element(document).scope();
                if(scope.user.showTour) {
                    if(scope.tourBackupDisabled) {
                        scope.tourBackupDisabled = false;
                    }
                    window.location.href = "#";
                }
            });
        function sortIDs(a, b) {
            return (a.id < b.id) ? -1 : ((a.id > b.id) ? 1 : 0);
        }

        function refreshSystemList() {
            PHD.appliance_sid = 1; // default to local
            console.log('set appliance id to ' + PHD.appliance_sid);
            var systems = PHD.Ajax.get('/api/systems');
            systems.done(function(data) {
                var appliances = data.appliance.sort(sortIDs);
                appliances.forEach(function(system, index) {
                    console.log("adding system " + system.name);
                    applianceSelect.options[index] = new Option(system.name, system.id);
                });
                if(serverEdit){
                    $("#id_system").val(associatedAppliance);
                }
            });
        }
        refreshSystemList();
    }

    function changePort() {
        if($protocol.val() == "nfs") {
            $port.val('2049');
        } else {
            $port.val('445');
        }
    }

    /*
     *   DOM Ready Function
     */
    $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });
        $protocol.on("change", function () {
            changePort();
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);

