var helpApp = angular.module('help', []);

helpApp.factory('helpFactory', function($http) {
    return {
        async: function(url, extra_params, callback) {
            var params = {};
            params.ts = Date.now();
            if (extra_params != null) {
                params = $.extend(params, extra_params);
            }
            var promise = $http({
                url: url,
                method: "get",
                params: params
            }).success(function(data) {
                console.log("success");
            }).error(function(){
                window.PHD.throwError({"result": [{"message": gettext("Cannot load help information.")}]});
                //PHD.hideLoadingIndicator(load);
            });  //1. this returns promise
            return promise;
        }
    };
});


helpApp.controller('HelpCtrl', ['$scope', '$rootScope', '$http', '$route', '$window', 'helpFactory', function($scope, $rootScope, $http, $route, $window, helpFactory) {

        // Help functions, specific variables.

        $scope.getSupportURL = function() {
            return "http://www.unitrends.com/support";
        };

        $scope.getOnlineHelpURL = function() {
            var url = "https://unitrends-support.force.com/";
            var free_article = "UnitrendsFree/s/article/";
            if ($rootScope.isGoogle) {
                url += free_article + "000003715";
            } else {
                url += "UnitrendsBackup/s/";
            }
            return url;

        };

        $scope.getKnowledgeBaseURL = function() {
            return $rootScope.isCE ? "https://unitrends-support.force.com/UnitrendsFree" : "http://www.unitrends.com/support/knowledge-base";
        };

        /*$scope.getVideoURL = function() {
            return "http://www.unitrends.com/resources/videos/tutorials";
        };*/


        $scope.tunnelNumber = 0;

        $scope.changeTunnelState = function() {

            var obj = {name: "Support Tunnel"};
            obj = JSON.stringify(obj);
            var url = "/api/commands";
            var load = null;
            // For opening the tunnel, need to tell user to wait as it takes awhile.
            if (!$scope.isTunnelOpen) {
                var message =  "Opening Support Tunnel. Please Wait...";
                load = PHD.showLoadingIndicator("body", true, message);
            } else {
                var message = "Closing Support Tunnel. Please Wait...";
                load = PHD.showLoadingIndicator("body", true, message);
            }
            //this is the call to actually open/close the support tunnel
            var resp = PHD.Ajax.post(url, obj, load);

            resp.done(function (data) {
                $scope.isTunnelOpen = !$scope.isTunnelOpen;
                // A successful return will be a string with "Creating tunnel: <number>\n...
                var r = /\d+/;
                var message = data.data;
                $scope.tunnelNumber = message.match(r);

                if (!$scope.isTunnelOpen) {
                    if ( $scope.tunnelNumber > 1 ) {
                                    PHD.alert("Support Tunnel " + $scope.tunnelNumber + " Successfully Closed.");
                                    $scope.tunnelNumber = 0;
                    } else {
                                    PHD.alert("Support Tunnel Successfully Closed.");
                                    $scope.tunnelNumber = 0;
                    }
                } else {
                    if ( $scope.tunnelNumber > 1 ) {
                                    PHD.alert("Support Tunnel " + $scope.tunnelNumber + " Successfully Opened.");
                    } else {
                                    PHD.alert("Support Tunnel Successfully Opened.");
                    }
                }
            });
        };

        $scope.getTunnelStatus = function() {
            var obj = {name: "Active Support Tunnel"};
            obj = JSON.stringify(obj);
            var url = "/api/commands";
            var resp = PHD.Ajax.post(url, obj);

            resp.done(function (data) {
                var tunnel = parseInt(data.data);
                $scope.isTunnelOpen = tunnel > 0;
		if ( $scope.tunnelNumber > 1 ) {
                $scope.$apply(function(){
                    $scope.tunnelNumber  = tunnel;
                });
		}
                console.log(tunnel);
            });
        };

        $scope.getFeedbackURL = function(){
            return FEEDBACK_URL + "/topic/0TO40000000PMPcGAO/Product Ideas";
        };

        $scope.getTourURL = function() {
            return "#";
        };
        $scope.startTour = function() {
            $scope.user.showTour = true;
            var loc = $window.location.href;
            // redirect to main page if not there already.
            if (!loc.match(/#\/$/)) {
                $window.location.href = "#";
            }
            $route.reload();
        };

        function displayAbout(data) {
            var data = data.appliance;
            for (var i = 0; i < data.length; i++) {
                obj = data[i];
                if (obj.local) {
                    PHD.miniWizard("app/help/about.html", "About", {
                        title: "About this Appliance",
                        dialogClass: "standalone",
                        width: 500,
                        height: "auto",
                        buttons: [
                            {
                                text: gettext("Close"),
                                click: function() {
                                    $(this).dialog("close");
                                }
                            }
                        ],
                        open: function(event, ui) {
                            $("#about_system_name").val(obj.name);
                            $("#about_system_host").val(obj.host);
                            $("#about_system_version").val(obj.version);
                            $("#about_asset_tag").val($rootScope.assetTag);

                        }
                    });
                    break;
                }
            }
        }

        $scope.runAbout = function() {
            helpFactory.async('/api/systems/', {}).then(function(data) {
                displayAbout(data.data);
            });
        };


        /*
         The Feedback Form
         */
        function displayFeedback(data) {
            var data = data.appliance;
            for (var i = 0; i < data.length; i++) {
                obj = data[i];
                if (obj.local) {
                    PHD.wizard("app/help/feedback.html", "Feedback", {
                        title: "Enhancement Request/Feedback",
                        dialogClass: "standalone",
                        width: 500,
                        height: 600
                    });
                    break;
                }
            }
        }

        /*
         Get the system info to display in the feedback email.
         */
        $scope.runFeedback = function() {
            helpFactory.async('/api/systems/', {}).then(function(data) {
                displayFeedback(data.data);
            });
        };

    }]

);
