var jobApp = angular.module('jobs', ['download', 'angulartics', 'angulartics.google.analytics','protect','ngDialog']);

jobApp.factory('jobFactory', function($http, DownloadService) {
    return {
        async: function(url, start_date, end_date, report_name, extra_params) {
            var report_name = report_name || null;
            var extra_params = extra_params || null;
            var params = {};
            params.ts = Date.now();
            if (extra_params != null) {
                params = $.extend(params, extra_params);
            }
            var promise = $http({
                url: url,
                method: "get",
                params: params
            }).success(function(data, status, headers) {
                console.log("success");
                // This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
                headers = headers();
                var contentType = headers['content-type'];
                if (contentType.indexOf('text/csv') > -1) {
                    var filename = report_name == 'null' ? 'report' + Date.now + ".csv" : report_name.replace(" ", "-") + ".csv";
                    var blob = new Blob([data], {type: contentType});
                    DownloadService.downloadFile(blob, filename);
                }
            }).error(function(){
                window.PHD.throwError({"result": [{"message": gettext("Requested job data cannot be exported.")}]});
                PHD.hideLoadingIndicator(load);
            });  //1. this returns promise
            return promise;
        }
    };
});

jobApp.controller('JobsCtrl', ['$scope', '$rootScope', '$http', 'Page', 'jobFactory', 'DownloadService', '$analytics','$protectService','ngDialog',
        function($scope, $rootScope, $http, Page, jobFactory, DownloadService, $analytics, $protectService,ngDialog) {

            $scope.currentTabIndex = 0;

            Page.setTitle('Jobs');

            $scope.$on('$viewContentLoaded', function() {
                processJobsPage(PHD, window, $rootScope, $scope, $analytics);
                processJobInventory(PHD, window);
            });

            $scope.exportHistory = function() {
                $analytics.eventTrack('Export History', {  category: 'Jobs', label: 'Jobs' });
                var load = PHD.showLoadingIndicator($(document));
                jobFactory.async("/api/jobs/history/?showFlat=true&sort=desc&format=csv", null, null, 'Job History', null).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                })
            };

            $scope.exportAudits = function() {
                $analytics.eventTrack('Export System', {  category: 'Jobs', label: 'Jobs' });
                var load = PHD.showLoadingIndicator($(document));
                jobFactory.async("/api/jobs/system/?format=csv", null, null, 'System Jobs', null).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                })
            };

            var rootNodeObj={};
            var rootNodeArray=[];
            var finalTreeDataArray = [];

            $scope.openBackupJobDialog = function(){
                $scope.selectedAssetType = INVENTORY_TYPE_FAMILY_CLIENT;
                $scope.selectedNodeList = [];
                $protectService.openBackupDialog($http,$scope,$rootScope,ngDialog, false);
            };

            $scope.editBackupJobDialog = function() {
                if (!$rootScope.isSelectedJobCopy) {
                    $protectService.openBackupDialog($http,$scope,$rootScope,ngDialog, true, $rootScope.selectedJobIdToEdit);
                }
            };

            $scope.jobHistorySave = function() {
                    $rootScope.jobHistoryLimit = $('#id_job_history_load').val();
                $scope.toggleModal();
                $(document).trigger("limit-changed", [JOB_TAB_HISTORY, $scope.jobHistoryLimit]);
            };

            $scope.systemJobHistorySave = function() {
                $rootScope.systemJobHistoryLimit = $('#id_system_job_history_load').val();
                $scope.toggleModal();
                $(document).trigger("limit-changed", [JOB_TAB_SYSTEM, $scope.systemJobHistoryLimit]);
            };


            /* Click Event Listener for open configure model dialog box  */
            $scope.modalShown=false;

            $scope.toggleModal = function(){
                if($scope.currentTabIndex === 2){
                    this.toggleHistoryModal();
                } else if ($scope.currentTabIndex === 3){
                    this.toggleSystemHistoryModal();
                }
            };

            $scope.toggleHistoryModal=function(){
                  $('#id_job_history_load').val($scope.jobHistoryLimit);
                  $scope.modalShown=!$scope.modalShown;
            };
            $scope.toggleSystemHistoryModal=function(){
                $('#id_system_job_history_load').val($scope.systemJobHistoryLimit);
                $scope.modalShown=!$scope.modalShown;
            };
        }]
);
