var fileLevelRecoveryModule = angular.module('fileLevelRecoveryModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics','ui.grid','ui.grid.selection']);

fileLevelRecoveryModule.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false
    });
}]);

fileLevelRecoveryModule.controller('fileLevelRecoveryCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', 'wizardService', '$analytics',
    function($scope, $rootScope, $http, $timeout, ngDialog, wizardService, $analytics) {

        var filePath="";
        var totalListItems = 0;
        $scope.flr_flag = "";
        $scope.synthCapable = selectedBackup.synth_capable?"synthesized-files":"files";
        $scope.FileLevelRecoveryOptionForm = $("#flr-file-option-form");
        $scope.isPhysicalServerRecovery = (selectedBackup.instance_id === undefined) ? true:false;
        $scope.system_id = selectedBackup.replicated == "true" ? selectedBackup.local_system_id : selectedBackup.sid;
        $scope.browseOnly = selectedBackup.client_os == "iSeries";              // for certain types of clients, block restore but allow browse (applies to only iSeries at this time).

        // NDMP variables
        $scope.ndmpRestoreTarget = {};
        $scope.ndmpRestoreTarget.asset = "";
        $scope.ndmpRestoreTarget.assetName = "";
        $scope.ndmpRestoreTarget.directory = "";
        $scope.ndmpTargets = [];
        $scope.ndmpTargetVolumes = [];
        $scope.ndmpRestoreTargetVolume = {};
        $scope.ndmpRestoreTargetVolume.instance_id = "";
        $scope.ndmpRestoreTargetVolume.assetName = "";

        $scope.initializeController = function () {
            if((selectedBackup.backup_id != undefined) && (selectedBackup.backup_id != null) && (selectedBackup.backup_id != "")){
                $scope.flr_flag = "Backup";
                $scope.initFileBrowserTree();
            }else if((selectedBackup.archive_id != undefined) && (selectedBackup.archive_id != null) && (selectedBackup.archive_id != "")){
                $scope.flr_flag = "BackupCopy";
                $scope.initFileBrowserTreeFLRBackupCopy();
            }
        };

        $scope.initializeRecoveryOptionTab = function(){
            $scope.getRestoreBackupTarget();
        };

        $scope.initializeNDMPRecoveryOptionTab = function() {
            $scope.getNDMPRestoreBackupTarget();
        };

        function validateFileBrowser(){
            $.validator.addMethod("isValidateFileBrowser", function(value, element) {
                    var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                    return listbox.length > 0;
                },
                "Select files to recover"
            );
        }

        /**********************************Wizard next steps.*********************/
        $scope.nextStepWizard = function($event) {
            if ($scope.browseOnly) {
                var message = selectedBackup.client_os + " backups may be browsed but not recovered using the web-based user interface.  " +
                              " To recover an " + selectedBackup.client_os + " backup, use the menu-based console interface.";
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'browseOnlyError',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            } else {
                if($scope.flr_flag === "Backup"){
                    if (wizardTitle == "File Browser") {//1st step for backup client
                        var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                        if(validateFileBrowserTab()){
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    }else if (wizardTitle == "Recovery Option") { //2nd step
                        $scope.postFileLevelRecoveryData(); //call to the POST API
                    }
                }else if($scope.flr_flag === "BackupCopy"){
                    if (wizardTitle == "File Browser") {//1st step for backup copy client
                        var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
                        if(validateFileBrowserTab()){
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    }else if (wizardTitle == "Backup Copy Option") { //2nd step
                        $scope.postFileLevelBackupCopyRecoveryData(); //call to the POST API
                    }
                }
            }
        };

        $scope.onConfirmError = function () {
            ngDialog.close('browseOnlyError');
        };

        /**************************************************************************/

        /* START: File browser tab for FLR-Bckup */
        $scope.initFileBrowserTree = function(){
            var parentId = " ";
            var backupSelectionObject =null;

            $('#fileBrowserTreeID').jstree({
                'plugins' : ['types','dnd'],
                'core' : {
                    'data' : {
                        'url' : function( node ){
                            parentId = encodeURIComponent(node.id);
                            if( node.id == "#" ){
                                return "/api/backups/" + $scope.synthCapable + "/" + selectedBackup.backup_id + "/?sid=" + $scope.system_id;
                            } else {
                                return "/api/backups/" + $scope.synthCapable + "/" + selectedBackup.backup_id + "/?&dir=" + parentId + "&sid=" + $scope.system_id;
                            }
                        },
                        'dataFilter' : function (data) {
                            var jsonData=JSON.parse(data);
                            jsonData = jsonData.data;

                            var treeArray = [];
                            var treeObj={};
                            if(jsonData !== null && jsonData !== undefined){
                                for( i =0 ; i<jsonData.length ; i++ ){

                                    var index = jsonData[i].name.indexOf("/");
                                    var nodeName = jsonData[i].name;
                                    var nodeId = jsonData[i].id;
                                    if(index != undefined && index !=null && index == -1 && (jsonData[i].type === "directory" || jsonData[i].type === "volume")){
                                        nodeName = jsonData[i].name + "/";
                                        nodeId = jsonData[i].id+"/";
                                    }
                                    jsonData[i].isBranch = true;
                                    treeObj = {"id":nodeId,"text":nodeName,icon:(jsonData[i].type === "directory" ||
                                    jsonData[i].type === "volume")? "icon-hv-folder icon" : "icon-file icon",
                                        state: { opened: false},"children":(jsonData[i].type === "directory" ||
                                        jsonData[i].type === "volume")? true: false};

                                    treeArray.push(treeObj);
                                }
                            }
                            var newdata1= JSON.stringify(treeArray);
                            return newdata1;
                        }
                    },
                    'multiple' : true,
                    'animation': true,
                    'error' : function(error) {$log.error('treeCtrl: error from js tree - ' + angular.toJson(error));},
                    'check_callback' : true,
                    'worker' : true,
                    'strings': {
                        'nodes': 'files/folders'
                    }
                }

            });

            $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
                var i, j, r = [];
                $scope.filePathArray = [];
                for(i = 0, j = item.selected.length; i < j; i++) {
                    $scope.filePathArray.push(item.selected[i]);
                }
            });
        };


        $scope.addItem = function(){
            console.log($scope.filePathArray);
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i=0; i<$scope.filePathArray.length;i++){
                addOption(listbox, $scope.filePathArray[i]);
            }
        };

        function addOption(selectbox,node){
            console.log(node);
            var optn = document.createElement("OPTION");
            optn.text = node;
            optn.value = node;

            var exists = false;
            $('#listboxId option').each(function(){
                if (this.value == node) {
                    exists = true;
                }
            });

            if(!exists && !angular.isUndefined(node) && node !== null){
                selectbox.options.add(optn);
                totalListItems = selectbox.options.length;
                document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
            }
        }

        $scope.removeItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            for(var i = 0; i < listbox.options.length; i++) {
                if(listbox.options[i].selected == true) {
                    try {listbox.remove(i, null);} catch(error) {
                        listbox.remove(i);
                    }
                }
            }
            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
        };

        $scope.removeAllItem=function(){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            listbox.innerHTML = ""
            totalListItems = listbox.options.length;
            document.getElementById("totalListItemId").innerHTML= "File Selected:"+totalListItems;
        };

        function validateFileBrowserTab(){
            validateFileBrowser();
            $("#flr-file-browser-form").validate({
                errorLabelContainer: ".errorlist",
                errorContainer: "#dialog-errors",
                showErrors: function(errorMap, errorList) {
                    $("#dialog-errors").find(".summary")
                        .html(gettext("Correct the following errors:"));
                    this.defaultShowErrors();
                },
                rules: {
                    inventoryListbox:{
                        isValidateFileBrowser:true
                    }
                },
                messages: {
                    isValidateFileBrowser:"Select files to recover"
                }
            });
            if((!$('#flr-file-browser-form').valid())) {
                return false;
            }else{
                return true;
            }
        }


        /* END: File browser tab */

        /* START: Recovery Options Tab */

        $scope.getRestoreBackupTarget=function(){
            var defaulatSelectedClientId = 1;
            $('#id_asset').empty();
            var resp = PHD.Ajax.get("/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&sid=" + $scope.system_id);
            resp.done(function(data) {
                console.log(data);
                defaulatSelectedClientId = parseInt(selectedBackup.client_id, 10);
                console.log(defaulatSelectedClientId);
                $.each(data.targets,function(i,target){
                    $('#id_asset').append("<option type="+target.client_os+" value="+target.client_id+">"+target.client_name+"</option>");
                });
                $('#id_asset').val(defaulatSelectedClientId);
            })
        };

        $scope.getNDMPRestoreBackupTarget = function() {
            var defaultSelectedClientId = 1;
            $('#id_host').empty();
            var resp = PHD.Ajax.get("/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&sid=" + $scope.system_id);
            resp.done(function(data) {
                console.log(data);
                var restore_targets = data.targets;
                defaultSelectedClientId = parseInt(selectedBackup.client_id, 10);
                console.log(defaultSelectedClientId);
                angular.forEach(restore_targets, function (value, key) {
                    var locationTempObj = {};
                    locationTempObj.value = value.client_id;
                    locationTempObj.name = value.client_name;
                    $scope.ndmpTargets.push(locationTempObj);
                });
                //$('#id_host').val(defaultSelectedClientId);
            })
        };

        $scope.ndmpTargetChange = function (ndmp_target_client_id) {
            if (ndmp_target_client_id !== null) {
                $scope.ndmpRestoreTarget.assetName = $.grep($scope.ndmpTargets, function (asset) {
                    return asset.value === ndmp_target_client_id;
                })[0].name;
            } else {
                $scope.ndmpRestoreTarget.assetName = "";
            }

            var url = "/api/restore/targets/?backup_id=" + selectedBackup.backup_id + "&ndmp_target_client_id=" + ndmp_target_client_id + "&replicated=";
            url += selectedBackup.replicated == 'true' ? "1&sid=" + selectedBackup.local_system_id : "0&sid=" + selectedBackup.sid;
            load = PHD.showLoadingIndicator("body", true, "Loading Target Volumes...");
            var resp = PHD.Ajax.get(url);

            resp.done(function(data) {
                PHD.hideLoadingIndicator(load);
                var restore_target_volumes = data.targets;
                if (restore_target_volumes && restore_target_volumes.length > 0) {
                    $scope.ndmpTargetVolumes = [];
                    angular.forEach(restore_target_volumes, function (value, key) {
                        var volumeTempObj = {};
                        volumeTempObj.value = value.instance_id;
                        volumeTempObj.name = value.name;
                        $scope.ndmpTargetVolumes.push(volumeTempObj);
                    });
                    //$scope.ndmpRestoreTarget.asset = $scope.ndmpTargets[0].value;
                    //$scope.ndmpRestoreTargetVolume.instance_id = $scope.ndmpTargetVolumes[0].value;
                    //$scope.ndmpRestoreTarget.assetName = $scope.ndmpTargets[0].name;

                }
            });
        };

        $scope.ndmpTargetVolumeChange = function () {
            if ($scope.ndmpRestoreTargetVolume.instance_id !== null) {
                $scope.ndmpRestoreTargetVolume.assetName = $.grep($scope.ndmpTargetVolumes, function (asset) {
                    return asset.value === $scope.ndmpRestoreTargetVolume.instance_id;
                })[0].name;
            } else {
                $scope.ndmpRestoreTargetVolume.assetName = "";
            }
        };


        // For add exclude patern.
        $scope.addExcludePatternRow = function() {
            var table = $("#exclusionTable")[0];
            var excludePattern = $("#id_add").val();

            if(excludePattern == ""){
                return;
            }
            var rowCount = table.rows.length;
            var row = table.insertRow(rowCount);
            var cell1 = row.insertCell(0);
            cell1.innerHTML= excludePattern;

            var cell2 = row.insertCell(1);
            cell2.innerHTML = '<span class="btn-remove-recipient"><i title="Remove" onclick="deleteRow(this)" class="icon-removesign"></i></span>';
            $("#id_add").val("");
        };

        $scope.postFileLevelRecoveryData = function() {
            var load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery...");
            // var backups = selectionObj.backups;
            var bids = selectedBackup.backup_id;
            PHD.appliance_sid = $scope.system_id;
            var restoreObj = {};
            restoreObj.backup_id = bids;

            $scope.isPhysicalServerRecovery = (selectedBackup.instance_id === undefined) ? true:false;

            // this needs to be done only in case of File level recovery
            restoreObj.client_id = $('#id_asset').val();
            if ( $scope.is_cisco_ucs_restore !== true && $scope.is_NDMP_restore !== true) {
                var target = {};
                target.flat = !$('#id_preserve_der').is(":checked");
                target.non_destructive = !$('#id_overwrt_file').is(":checked");
                target.newer = $('#id_newer_file_restore').is(":checked");
                target.today = $('#id_file_dates').is(":checked");
                target.unix = $('#idunix_conver').is(":checked");
                target.directory = $('#id_directory').val();
                restoreObj.target = target;
                restoreObj.before_cmd = $('#id_pre_restore').val();
                restoreObj.after_cmd = $('#id_post_restore').val();
            }
            if($scope.synthCapable === "synthesized-files") {
                restoreObj.synthesis = true;
            } else {
                restoreObj.synthesis = false;
            }
            if($scope.is_NDMP_restore === true) {
                restoreObj.target = {};
                restoreObj.target.volume = "";
                restoreObj.target.directory = "";
                restoreObj.client_id = $scope.ndmpRestoreTarget.asset;
                restoreObj.target.volume = $scope.ndmpRestoreTargetVolume.assetName;
                restoreObj.target.directory = $scope.ndmpRestoreTarget.directory;
                restoreObj.backup_id = (selectedBackup.backup_id);
            }
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var includeListArray = [];
            var excludeListArray = [];

            for(var i=0; i< listbox.length; i++){
                includeListArray.push(listbox[i].text);
            }
            console.log(includeListArray);
            restoreObj.includes = includeListArray;

            if ( $scope.is_cisco_ucs_restore !== true && $scope.is_NDMP_restore !== true) {
                var data = document.getElementById("exclusionTable").rows;
                for (var j = 0; j < data.length; j++) {
                    var excludeList = document.getElementById("exclusionTable").rows[j].cells[0].innerHTML;
                    excludeList = excludeList.replace(/[\r\n]/g, "").trim();
                    if (excludeList != '\n' && excludeList != '\r' && excludeList != '\t' && excludeList != "") {
                        if (excludeList.length != 0) {
                            excludeListArray.push(excludeList);
                        }
                    }
                }
                restoreObj.excludes = excludeListArray;
            }
            restoreObj = JSON.stringify(restoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            //var url = "/api/restore/files/" + sid;
            url = "/api/restore/full/" + sid;

            $http ({
                method: 'post',
                url : url,
                data: restoreObj
            }).success(function(data){
                /*data = {};
                 PHD.hideLoadingIndicator(load);

                 ngDialog.open({
                 dialogType:'Information',
                 scope:$scope,
                 dialogMessage:"File level recovery job successfully started."
                 });*/
                PHD.hideLoadingIndicator(load);
                data = {};
                data.result = [{'message': 'File level recovery job successfully started.'}];
                var options = {};
                if (window.location.href.indexOf("#/protect") !== -1 || window.location.href.indexOf("#/jobs") !== -1) {
                    options.buttons = [
                        {
                            text: gettext("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }

                PHD.throwNotice(data, options);

                ngDialog.close('create-filelevel-recovery-dialog');

            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
        };
        /* END: Recovery Options Tab */


        /* START: Call POST api for file level backup copy recovery data */
        $scope.postFileLevelBackupCopyRecoveryData = function(){
            var load = PHD.showLoadingIndicator("body", true, "Starting File Level Recovery...");
            PHD.appliance_sid = $scope.system_id;
            var backupCopyRestoreObj = {};
            backupCopyRestoreObj.archive_ids = selectedBackup.archive_id; //selectedBackup.archive_ids;
            backupCopyRestoreObj.association = "dr"; //association  association
            //backupCopyRestoreObj.instance_id = "";
            //backupCopyRestoreObj.instance_id = selectedBackup.instance_id;  // TO DO: when instance_id corrected


            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
            var includes_list = [];
            for(var i=0; i< listbox.length; i++){
                includes_list[i] = listbox[i].text;
            }
            backupCopyRestoreObj.inclusions = includes_list;
            backupCopyRestoreObj.exclusions = [];
            backupCopyRestoreObj.restore_backup = true;

            backupCopyRestoreObj = JSON.stringify(backupCopyRestoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            url = "/api/restore/archive/files/" + sid;


            $http ({
                method: 'post',
                url : url,
                data: backupCopyRestoreObj
            }).success(function(data){
                PHD.hideLoadingIndicator(load);
                data = {};
                data.result = [{'message': 'File level recovery job successfully started.'}];
                var options = {};
                if (window.location.href.indexOf("#/protect") !== -1 || window.location.href.indexOf("#/jobs") !== -1) {
                    options.buttons = [
                        {
                            text: gettext("View FLR"),
                            click: function () {
                                window.location.href = "#/recover/#1";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }

                PHD.throwNotice(data, options);
                ngDialog.close('create-filelevel-recovery-dialog');

            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
        };


        /* END: Call POST api for file level backup copy recovery data  */

        /* START: File browser tab for FLR-Bckup copy - GET Api */
        $scope.initFileBrowserTreeFLRBackupCopy = function(){
            var parentId = " ";
            var file_id = selectedBackup.archive_id;     //selectedBackup.backup_id

            $('#fileBrowserTreeID').jstree({
                'plugins' : ['types','dnd'],
                'core' : {
                    'data' : {
                        'url' : function( node ){
                            parentId = encodeURIComponent(node.id);
                            if( node.id == "#" ){
                                return "/api/archive/files/" + file_id + "/?sid=" + $scope.system_id;
                            } else {
                                return "/api/archive/files/" + file_id + "/?&dir=" + parentId + "&sid=" + $scope.system_id;
                            }
                        },
                        'dataFilter' : function (data) {
                            var jsonData=JSON.parse(data);
                            jsonData = jsonData.data;

                            var treeArray = [];
                            var treeObj={};
                            if(jsonData !== null && jsonData !== undefined){
                                for( i =0 ; i<jsonData.length ; i++ ){

                                    var index = jsonData[i].name.indexOf("/");
                                    var nodeName = jsonData[i].name;
                                    var nodeId = jsonData[i].id;
                                    if(index != undefined && index !=null && index == -1 && (jsonData[i].type === "directory" || jsonData[i].type === "volume")){
                                        nodeName = jsonData[i].name+"/";
                                    }
                                    jsonData[i].isBranch = true;
                                    treeObj = {"id":nodeId,"text":nodeName,icon:(jsonData[i].type === "directory" ||
                                    jsonData[i].type === "volume")? "icon-hv-folder icon" : "icon-file icon",
                                        state: { opened: false},"children":(jsonData[i].type === "directory" ||
                                        jsonData[i].type === "volume")? true: false};

                                    treeArray.push(treeObj);
                                }
                            }
                            var newdata1= JSON.stringify(treeArray);
                            return newdata1;
                        }
                    },
                    'multiple' : true,
                    'animation': true,
                    'error' : function(error) {$log.error('treeCtrl: error from js tree - ' + angular.toJson(error));},
                    'check_callback' : true,
                    'worker' : true
                }
            });

            $('#fileBrowserTreeID').on('select_node.jstree',function(e,item) {
                var i, j, r = [];
                $scope.filePathArray = [];
                for(i = 0, j = item.selected.length; i < j; i++) {
                    $scope.filePathArray.push(item.selected[i]);
                }
            });
        };

        $(document).on('dnd_move.vakata', function (e, data) {
            var t = $(data.event.target);
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    data.helper.find('.jstree-icon').removeClass('jstree-er').addClass('jstree-ok');
                }
                else {
                    data.helper.find('.jstree-icon').removeClass('jstree-ok').addClass('jstree-er');
                }
            }

        }).on('dnd_stop.vakata', function (e, data) {
            console.log(data.data.nodes);
            var t = $(data.event.target);
            if(!t.closest('.jstree').length) {
                if(t.closest('.validate-filebrowser').length) {
                    console.log(data.data.nodes);
                    addDragNodesInList(data.data.nodes);
                    $(data.element).clone().appendTo(t.closest('.validate-filebrowser'));
                }
            }
        });

		function addDragNodesInList(textArray){
            var listbox = document.FileLevelRecoveryBrowserForm.inventoryListbox;
			var newText = "";  
			var backSlashChar = "/";
            for(var i=0;i<textArray.length;i++){
                if(textArray[i] != null){
                    addOption(listbox,textArray[i]);
		  		}
	         }
         }

    }]);

/* function for delete row from exclusion list in recovery option tab */
function deleteRow(r){
    var i = r.parentNode.parentNode.parentNode.rowIndex;
    document.getElementById("exclusionTable").deleteRow(i);
}

